@@ -1,8 +1,8 @@
 package Number::Format;
 
-# Minimum version is 5.8.0.  May work on earlier versions, but not
-# supported on any version older than 5.8.
-require 5.008;
+# Minimum version is 5.10.0.  May work on earlier versions, but not
+# supported on any version older than 5.10.  Hack this line at your own risk:
+require 5.010;
 
 use strict;
 use warnings;
@@ -142,8 +142,8 @@ higher are not implemented because of integer overflows on 32-bit
 systems.
 
 The only restrictions on C<DECIMAL_POINT> and C<THOUSANDS_SEP> are that
-they must not be digits, must not be identical, and must each be one
-character.  There are no restrictions on C<INT_CURR_SYMBOL>.
+they must not be digits and must not be identical.  There are no
+restrictions on C<INT_CURR_SYMBOL>.
 
 For example, a German user might include this in their code:
 
@@ -178,7 +178,7 @@ you can use the tag C<:all>.
 use strict;
 use Exporter;
 use Carp;
-use POSIX;
+use POSIX qw(localeconv);
 use base qw(Exporter);
 
 our @EXPORT_SUBS =
@@ -211,7 +211,7 @@ our %EXPORT_TAGS = ( subs             => \@EXPORT_SUBS,
                      other_vars       => \@EXPORT_OTHER,
                      all              => \@EXPORT_ALL );
 
-our $VERSION = '1.72';
+our $VERSION = '1.75';
 
 # Refer to http://www.opengroup.org/onlinepubs/007908775/xbd/locale.html
 # for more details about the POSIX variables
@@ -284,6 +284,17 @@ our $DEFAULT_LOCALE = { (
                          gibi_suffix       => $GIBI_SUFFIX,
                         ) };
 
+#
+# On Windows, the POSIX localeconv() call returns illegal negative
+# numbers for some values, seemingly attempting to indicate null.  The
+# following list indicates the values for which this has been
+# observed, and for which the values should be stripped out of
+# localeconv().
+#
+our @IGNORE_NEGATIVE = qw( frac_digits int_frac_digits
+                           n_cs_precedes n_sep_by_space n_sign_posn
+                           p_xs_precedes p_sep_by_space p_sign_posn );
+
 #
 # Largest integer a 32-bit Perl can handle is based on the mantissa
 # size of a double float, which is up to 53 bits.  While we may be
@@ -311,7 +322,9 @@ use constant MAX_INT => 2**53;
 my $DefaultObject;
 sub _get_self
 {
-    unless (ref $_[0])
+    # Not calling $_[0]->isa because that may result in unblessed
+    # reference error
+    unless (ref $_[0] && UNIVERSAL::isa($_[0], "Number::Format"))
     {
         $DefaultObject ||= new Number::Format();
         unshift (@_, $DefaultObject);
@@ -376,6 +389,23 @@ sub _get_multipliers
     }
 }
 
+##----------------------------------------------------------------------
+
+# _complain_undef displays a warning message on STDERR and is called
+# when a subroutine has been invoked with an undef value.  A warning
+# message is printed if the calling environment has "uninitialized"
+# warnings enabled.
+
+sub _complain_undef
+{
+    my @stack;
+    my($sub, $bitmask) = (caller(1))[3,9];
+    my $offset = $warnings::Offsets{"uninitialized"};
+    carp "Use of uninitialized value in call to $sub"
+         if vec($bitmask, $offset, 1);
+}
+
+
 ###---------------------------------------------------------------------
 
 =head1 METHODS
@@ -409,7 +439,14 @@ sub new
     # my $locale        = setlocale(LC_ALL, "");
     my $locale_values = localeconv();
 
-    my $arg;
+    # Strip out illegal negative values from the current locale
+    foreach ( @IGNORE_NEGATIVE )
+    {
+        if (defined($locale_values->{$_}) && $locale_values->{$_} eq '-1')
+        {
+            delete $locale_values->{$_};
+        }
+    }
 
     while(my($arg, $default) = each %$DEFAULT_LOCALE)
     {
@@ -470,9 +507,25 @@ variables, use C<format_number()> instead.
 sub round
 {
     my ($self, $number, $precision) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     $precision = $self->{decimal_digits} unless defined $precision;
     $precision = 2 unless defined $precision;
-    $number    = 0 unless defined $number;
+
+    croak("precision must be integer")
+        unless int($precision) == $precision;
+
+    if (ref($number) && $number->isa("Math::BigFloat"))
+    {
+        my $rounded = $number->copy();
+        $rounded->precision(-$precision);
+        return $rounded;
+    }
 
     my $sign       = $number <=> 0;
     my $multiplier = (10 ** $precision);
@@ -525,6 +578,13 @@ C<DECIMAL_POINT> instead of ',' and '.' respectively.
 sub format_number
 {
     my ($self, $number, $precision, $trailing_zeroes, $mon) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     $self->_check_seps();       # first make sure the SEP variables are valid
 
     my($thousands_sep, $decimal_point) =
@@ -575,8 +635,8 @@ sub format_number
         $integer = join($thousands_sep,
                         grep {$_ ne ''} split(/(...)/, $integer));
 
-        # Strip off leading zeroes and/or comma
-        $integer =~ s/^0+\Q$thousands_sep\E?//;
+        # Strip off leading zeroes and optional thousands separator
+        $integer =~ s/^0+(?:\Q$thousands_sep\E)?//;
     }
     $integer = '0' if $integer eq '';
 
@@ -604,6 +664,13 @@ but formatting will occur whether or not the number is negative.
 sub format_negative
 {
     my($self, $number, $format) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     $format = $self->{neg_format} unless defined $format;
     croak "Letter x must be present in picture in format_negative()"
         unless $format =~ /x/;
@@ -647,6 +714,15 @@ to avoid this problem, set NEG_FORMAT to "x".
 sub format_picture
 {
     my ($self, $number, $picture) = _get_self @_;
+
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
+    croak "Picture not defined" unless defined($picture);
+
     $self->_check_seps();
 
     # Handle negative numbers
@@ -768,6 +844,12 @@ sub format_price
 {
     my ($self, $number, $precision, $curr_symbol) = _get_self @_;
 
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     # Determine what the monetary symbol should be
     $curr_symbol = $self->{int_curr_symbol}
         if (!defined($curr_symbol) || lc($curr_symbol) eq "int_curr_symbol");
@@ -990,6 +1072,12 @@ sub format_bytes
 {
     my ($self, $number, @options) = _get_self @_;
 
+    unless (defined($number))
+    {
+        _complain_undef();
+        $number = 0;
+    }
+
     croak "Negative number not allowed in format_bytes"
         if $number < 0;
 
@@ -1133,6 +1221,13 @@ multiple of that value) as appropriate.  Examples:
 sub unformat_number
 {
     my ($self, $formatted, %options) = _get_self @_;
+
+    unless (defined($formatted))
+    {
+        _complain_undef();
+        $formatted = "";
+    }
+
     $self->_check_seps();
     return undef unless $formatted =~ /\d/; # require at least one digit
 
@@ -1192,6 +1287,12 @@ sub unformat_number
 
 =back
 
+=head1 CAVEATS
+
+Some systems, notably OpenBSD, may have incomplete locale support.
+Using this module together with L<setlocale(3)> in OpenBSD may therefore
+not produce the intended results.
+
 =head1 BUGS
 
 No known bugs at this time.  Report bugs using the CPAN request
